@@ -75,8 +75,9 @@ module Language.Haskell.Exts.Annotated.Syntax (
     Safety(..), CallConv(..),
 
     -- * Pragmas
-    OptionPragma(..), Tool(..),
+    ModulePragma(..), Tool(..),
     Rule(..), RuleVar(..), Activation(..),
+    Annotation(..),
 
     -- * Builtin names
 
@@ -110,21 +111,6 @@ import Data.Generics (Data(..),Typeable(..))
 #endif
 #endif
 
--- import Language.Haskell.Exts.SrcLoc
--- import Control.Monad.Instances  -- just for Functor (,) a
-
-{-- | A position in the source.
-data SrcLoc = SrcLoc {
-        srcFilename :: String,
-        srcLine :: Int,
-        srcColumn :: Int
-        }
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
--}
 
 -- | The name of a Haskell module.
 data ModuleName l = ModuleName l String
@@ -137,15 +123,14 @@ data ModuleName l = ModuleName l String
 -- | Constructors with special syntax.
 -- These names are never qualified, and always refer to builtin type or
 -- data constructors.
-
 data SpecialCon l
-    = UnitCon l           -- ^ unit type and data constructor @()@
-    | ListCon l           -- ^ list type constructor @[]@
-    | FunCon  l           -- ^ function type constructor @->@
-    | TupleCon l Boxed Int    -- ^ /n/-ary tuple type and data
-                                  --   constructors @(,)@ etc, possibly boxed @(\#,\#)@
-    | Cons l              -- ^ list data constructor @(:)@
-    | UnboxedSingleCon l  -- ^ unboxed singleton tuple constructor @(\# \#)@
+    = UnitCon l             -- ^ unit type and data constructor @()@
+    | ListCon l             -- ^ list type constructor @[]@
+    | FunCon  l             -- ^ function type constructor @->@
+    | TupleCon l Boxed Int  -- ^ /n/-ary tuple type and data
+                            --   constructors @(,)@ etc, possibly boxed @(\#,\#)@
+    | Cons l                -- ^ list data constructor @(:)@
+    | UnboxedSingleCon l    -- ^ unboxed singleton tuple constructor @(\# \#)@
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -217,12 +202,12 @@ data CName l
 
 -- | A complete Haskell source module.
 data Module l
-    = Module l (Maybe (ModuleHead l)) [OptionPragma l] [ImportDecl l] [Decl l]
+    = Module l (Maybe (ModuleHead l)) [ModulePragma l] [ImportDecl l] [Decl l]
     -- ^ an ordinary Haskell module
-    | XmlPage l (ModuleName l) [OptionPragma l] (XName l) [XAttr l] (Maybe (Exp l)) [Exp l]
+    | XmlPage l (ModuleName l) [ModulePragma l] (XName l) [XAttr l] (Maybe (Exp l)) [Exp l]
     -- ^ a module consisting of a single XML document. The ModuleName never appears in the source
     --   but is needed for semantic purposes, it will be the same as the file name.
-    | XmlHybrid l (Maybe (ModuleHead l)) [OptionPragma l] [ImportDecl l] [Decl l]
+    | XmlHybrid l (Maybe (ModuleHead l)) [ModulePragma l] [ImportDecl l] [Decl l]
                 (XName l) [XAttr l] (Maybe (Exp l)) [Exp l]
     -- ^ a hybrid module combining an XML document with an ordinary module
 #ifdef __GLASGOW_HASKELL__
@@ -250,18 +235,18 @@ data ExportSpecList l
 
 -- | An item in a module's export specification.
 data ExportSpec l
-     = EVar l (QName l)           -- ^ variable
-     | EAbs l (QName l)           -- ^ @T@:
-                                  -- a class or datatype exported abstractly,
-                                  -- or a type synonym.
-     | EThingAll l (QName l)      -- ^ @T(..)@:
-                                  -- a class exported with all of its methods, or
-                                  -- a datatype exported with all of its constructors.
-     | EThingWith l (QName l) [CName l]   -- ^ @T(C_1,...,C_n)@:
-                                          -- a class exported with some of its methods, or
-                                          -- a datatype exported with some of its constructors.
-     | EModuleContents l (ModuleName l)   -- ^ @module M@:
-                                          -- re-export a module.
+     = EVar l (QName l)                 -- ^ variable
+     | EAbs l (QName l)                 -- ^ @T@:
+                                        --   a class or datatype exported abstractly,
+                                        --   or a type synonym.
+     | EThingAll l (QName l)            -- ^ @T(..)@:
+                                        --   a class exported with all of its methods, or
+                                        --   a datatype exported with all of its constructors.
+     | EThingWith l (QName l) [CName l] -- ^ @T(C_1,...,C_n)@:
+                                        --   a class exported with some of its methods, or
+                                        --   a datatype exported with some of its constructors.
+     | EModuleContents l (ModuleName l) -- ^ @module M@:
+                                        --   re-export a module.
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -300,15 +285,15 @@ data ImportSpecList l
 -- | An import specification, representing a single explicit item imported
 --   (or hidden) from a module.
 data ImportSpec l
-     = IVar l (Name l)            -- ^ variable
-     | IAbs l (Name l)            -- ^ @T@:
-                                  -- the name of a class, datatype or type synonym.
-     | IThingAll l (Name l)       -- ^ @T(..)@:
-                                  -- a class imported with all of its methods, or
-                                  -- a datatype imported with all of its constructors.
+     = IVar l (Name l)                  -- ^ variable
+     | IAbs l (Name l)                  -- ^ @T@:
+                                        --   the name of a class, datatype or type synonym.
+     | IThingAll l (Name l)             -- ^ @T(..)@:
+                                        --   a class imported with all of its methods, or
+                                        --   a datatype imported with all of its constructors.
      | IThingWith l (Name l) [CName l]  -- ^ @T(C_1,...,C_n)@:
-                                        -- a class imported with some of its methods, or
-                                        -- a datatype imported with some of its constructors.
+                                        --   a class imported with some of its methods, or
+                                        --   a datatype imported with some of its constructors.
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -366,7 +351,6 @@ data Decl l
      -- ^ A foreign import declaration
      | ForExp       l (CallConv l)                    (Maybe String) (Name l) (Type l)
      -- ^ A foreign export declaration
-
      | RulePragmaDecl   l [Rule l]
      -- ^ A RULES pragma
      | DeprPragmaDecl   l [([Name l], String)]
@@ -375,18 +359,37 @@ data Decl l
      -- ^ A WARNING pragma
      | InlineSig        l Bool (Maybe (Activation l)) (QName l)
      -- ^ An INLINE pragma
+     | InlineConlikeSig l      (Maybe (Activation l)) (QName l)
+     -- ^ An INLINE CONLIKE pragma
      | SpecSig          l                             (QName l) [Type l]
      -- ^ A SPECIALISE pragma
      | SpecInlineSig    l Bool (Maybe (Activation l)) (QName l) [Type l]
      -- ^ A SPECIALISE INLINE pragma
      | InstSig          l      (Maybe (Context l))    (InstHead l)
      -- ^ A SPECIALISE instance pragma
+     | AnnPragma        l (Annotation l)
+     -- ^ An ANN pragma
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
   deriving (Eq,Ord,Show)
 #endif
 
+-- | An annotation through an ANN pragma.
+data Annotation l
+    = Ann       l (Name l)  (Exp l)
+    -- ^ An annotation for a declared name.
+    | TypeAnn   l (Name l)  (Exp l)
+    -- ^ An annotation for a declared type.
+    | ModuleAnn l           (Exp l)
+    -- ^ An annotation for the defining module.
+#ifdef __GLASGOW_HASKELL__
+  deriving (Eq,Ord,Show,Typeable,Data)
+#else
+  deriving (Eq,Ord,Show)
+#endif
+
+
 -- | A flag stating whether a declaration is a data or newtype declaration.
 data DataOrNew l = DataType l | NewType l
 #ifdef __GLASGOW_HASKELL__
@@ -446,7 +449,15 @@ data IPBind l = IPBind l (IPName l) (Exp l)
 -- | Clauses of a function binding.
 data Match l
      = Match l      (Name l) [Pat l]         (Rhs l) {-where-} (Maybe (Binds l))
-     | InfixMatch l (Pat l) (Name l) (Pat l) (Rhs l) {-where-} (Maybe (Binds l))
+        -- ^ A clause defined with prefix notation, i.e. the function name 
+        --  followed by its argument patterns, the right-hand side and an
+        --  optional where clause.
+     | InfixMatch l (Pat l) (Name l) [Pat l] (Rhs l) {-where-} (Maybe (Binds l))
+        -- ^ A clause defined with infix notation, i.e. first its first argument
+        --  pattern, then the function name, then its following argument(s),
+        --  the right-hand side and an optional where clause.
+        --  Note that there can be more than two arguments to a function declared
+        --  infix, hence the list of pattern arguments.
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -523,8 +534,6 @@ data InstDecl l
             -- ^ an associated data type implementation
     | InsGData  l (DataOrNew l) (Type l) (Maybe (Kind l)) [GadtDecl l] (Maybe (Deriving l))
             -- ^ an associated data type implemented using GADT style
-    | InsInline l Bool (Maybe (Activation l)) (QName l)
-            -- ^ an INLINE pragma
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -573,7 +582,7 @@ data Type l
         (Maybe (Context l))
         (Type l)                                -- ^ qualified type
      | TyFun   l (Type l) (Type l)              -- ^ function type
-     | TyTuple l Boxed [Type l]             -- ^ tuple type, possibly boxed
+     | TyTuple l Boxed [Type l]                 -- ^ tuple type, possibly boxed
      | TyList  l (Type l)                       -- ^ list syntax, e.g. [a], as opposed to [] a
      | TyApp   l (Type l) (Type l)              -- ^ application of a type constructor
      | TyVar   l (Name l)                       -- ^ type variable
@@ -657,9 +666,9 @@ data Asst l
 #endif
 
 -- | /literal/
--- Values of this type hold the abstract value of the literal, not the
+-- Values of this type hold the abstract value of the literal, along with the
 -- precise string representation used.  For example, @10@, @0o12@ and @0xa@
--- have the same representation.
+-- have the same value representation, but each carry a different string representation.
 data Literal l
     = Char       l Char     String     -- ^ character literal
     | String     l String   String     -- ^ string literal
@@ -679,71 +688,68 @@ data Literal l
 
 -- | Haskell expressions.
 data Exp l
-    = Var l (QName l)                 -- ^ variable
-    | IPVar l (IPName l)              -- ^ implicit parameter variable
-    | Con l (QName l)                 -- ^ data constructor
-    | Lit l (Literal l)               -- ^ literal constant
-    | InfixApp l (Exp l) (QOp l) (Exp l)     -- ^ infix application
-    | App l (Exp l) (Exp l)                  -- ^ ordinary application
-    | NegApp l (Exp l)                -- ^ negation expression @-/exp/@ (unary minus)
-    | Lambda l [Pat l] (Exp l)  -- ^ lambda expression
-    | Let l (Binds l) (Exp l)             -- ^ local declarations with @let@ ... @in@ ...
-    | If l (Exp l) (Exp l) (Exp l)            -- ^ @if@ /exp/ @then@ /exp/ @else@ /exp/
-    | Case l (Exp l) [Alt l]            -- ^ @case@ /exp/ @of@ /alts/
-    | Do l [Stmt l]                 -- ^ @do@-expression:
-                                    -- the last statement in the list
-                                    -- should be an expression.
-    | MDo l [Stmt l]                -- ^ @mdo@-expression
-    | Tuple l [Exp l]               -- ^ tuple expression
-    | TupleSection l [Maybe (Exp l)]  -- ^ tuple section expression, e.g. @(,,3)@
-    | List l [Exp l]                -- ^ list expression
-    | Paren l (Exp l)                 -- ^ parenthesised expression
-    | LeftSection l (Exp l) (QOp l)       -- ^ left section @(@/exp/ /qop/@)@
-    | RightSection l (QOp l) (Exp l)      -- ^ right section @(@/qop/ /exp/@)@
-    | RecConstr l (QName l) [FieldUpdate l]
-                                    -- ^ record construction expression
-    | RecUpdate l (Exp l)   [FieldUpdate l]
-                                    -- ^ record update expression
-    | EnumFrom l (Exp l)              -- ^ unbounded arithmetic sequence,
-                                    -- incrementing by 1: @[from ..]@
-    | EnumFromTo l (Exp l) (Exp l)        -- ^ bounded arithmetic sequence,
-                                    -- incrementing by 1 @[from .. to]@
-    | EnumFromThen l (Exp l) (Exp l)      -- ^ unbounded arithmetic sequence,
-                                    -- with first two elements given @[from, then ..]@
+    = Var l (QName l)                       -- ^ variable
+    | IPVar l (IPName l)                    -- ^ implicit parameter variable
+    | Con l (QName l)                       -- ^ data constructor
+    | Lit l (Literal l)                     -- ^ literal constant
+    | InfixApp l (Exp l) (QOp l) (Exp l)    -- ^ infix application
+    | App l (Exp l) (Exp l)                 -- ^ ordinary application
+    | NegApp l (Exp l)                      -- ^ negation expression @-/exp/@ (unary minus)
+    | Lambda l [Pat l] (Exp l)              -- ^ lambda expression
+    | Let l (Binds l) (Exp l)               -- ^ local declarations with @let@ ... @in@ ...
+    | If l (Exp l) (Exp l) (Exp l)          -- ^ @if@ /exp/ @then@ /exp/ @else@ /exp/
+    | Case l (Exp l) [Alt l]                -- ^ @case@ /exp/ @of@ /alts/
+    | Do l [Stmt l]                         -- ^ @do@-expression:
+                                            --   the last statement in the list
+                                            --   should be an expression.
+    | MDo l [Stmt l]                        -- ^ @mdo@-expression
+    | Tuple l [Exp l]                       -- ^ tuple expression
+    | TupleSection l [Maybe (Exp l)]        -- ^ tuple section expression, e.g. @(,,3)@
+    | List l [Exp l]                        -- ^ list expression
+    | Paren l (Exp l)                       -- ^ parenthesised expression
+    | LeftSection l (Exp l) (QOp l)         -- ^ left section @(@/exp/ /qop/@)@
+    | RightSection l (QOp l) (Exp l)        -- ^ right section @(@/qop/ /exp/@)@
+    | RecConstr l (QName l) [FieldUpdate l] -- ^ record construction expression
+    | RecUpdate l (Exp l)   [FieldUpdate l] -- ^ record update expression
+    | EnumFrom l (Exp l)                    -- ^ unbounded arithmetic sequence,
+                                            --   incrementing by 1: @[from ..]@
+    | EnumFromTo l (Exp l) (Exp l)          -- ^ bounded arithmetic sequence,
+                                            --   incrementing by 1 @[from .. to]@
+    | EnumFromThen l (Exp l) (Exp l)        -- ^ unbounded arithmetic sequence,
+                                            --   with first two elements given @[from, then ..]@
     | EnumFromThenTo l (Exp l) (Exp l) (Exp l)
-                                -- ^ bounded arithmetic sequence,
-                                    -- with first two elements given @[from, then .. to]@
-    | ListComp l (Exp l) [QualStmt l]     -- ^ ordinary list comprehension
-    | ParComp  l (Exp l) [[QualStmt l]]    -- ^ parallel list comprehension
-    | ExpTypeSig l (Exp l) (Type l)
-                                    -- ^ expression with explicit type signature
+                                            -- ^ bounded arithmetic sequence,
+                                            --   with first two elements given @[from, then .. to]@
+    | ListComp l (Exp l) [QualStmt l]       -- ^ ordinary list comprehension
+    | ParComp  l (Exp l) [[QualStmt l]]     -- ^ parallel list comprehension
+    | ExpTypeSig l (Exp l) (Type l)         -- ^ expression with explicit type signature
 
-    | VarQuote l (QName l)            -- ^ @'x@ for template haskell reifying of expressions
-    | TypQuote l (QName l)            -- ^ @''T@ for template haskell reifying of types
-    | BracketExp l (Bracket l)        -- ^ template haskell bracket expression
-    | SpliceExp l (Splice l)          -- ^ template haskell splice expression
-    | QuasiQuote l String String      -- ^ quasi-quotaion: @[$/name/| /string/ |]@
+    | VarQuote l (QName l)                  -- ^ @'x@ for template haskell reifying of expressions
+    | TypQuote l (QName l)                  -- ^ @''T@ for template haskell reifying of types
+    | BracketExp l (Bracket l)              -- ^ template haskell bracket expression
+    | SpliceExp l (Splice l)                -- ^ template haskell splice expression
+    | QuasiQuote l String String            -- ^ quasi-quotaion: @[$/name/| /string/ |]@
 
 -- Hsx
     | XTag l (XName l) [XAttr l] (Maybe (Exp l)) [Exp l]
-                                -- ^ xml element, with attributes and children
+                                            -- ^ xml element, with attributes and children
     | XETag l (XName l) [XAttr l] (Maybe (Exp l))
-                                -- ^ empty xml element, with attributes
-    | XPcdata l String            -- ^ PCDATA child element
-    | XExpTag l (Exp l)               -- ^ escaped haskell expression inside xml
+                                            -- ^ empty xml element, with attributes
+    | XPcdata l String                      -- ^ PCDATA child element
+    | XExpTag l (Exp l)                     -- ^ escaped haskell expression inside xml
 
 -- Pragmas
-    | CorePragma l      String (Exp l)     -- ^ CORE pragma
-    | SCCPragma  l      String (Exp l)     -- ^ SCC pragma
+    | CorePragma l      String (Exp l)      -- ^ CORE pragma
+    | SCCPragma  l      String (Exp l)      -- ^ SCC pragma
     | GenPragma  l      String (Int, Int) (Int, Int) (Exp l)
-                                        -- ^ GENERATED pragma
+                                            -- ^ GENERATED pragma
 
 -- Arrows
-    | Proc            l (Pat l) (Exp l)  -- ^ arrows proc: @proc@ /pat/ @->@ /exp/
-    | LeftArrApp      l (Exp l) (Exp l)  -- ^ arrow application (from left): /exp/ @-<@ /exp/
-    | RightArrApp     l (Exp l) (Exp l)  -- ^ arrow application (from right): /exp/ @>-@ /exp/
-    | LeftArrHighApp  l (Exp l) (Exp l)  -- ^ higher-order arrow application (from left): /exp/ @-<<@ /exp/
-    | RightArrHighApp l (Exp l) (Exp l)  -- ^ higher-order arrow application (from right): /exp/ @>>-@ /exp/
+    | Proc            l (Pat l) (Exp l)     -- ^ arrows proc: @proc@ /pat/ @->@ /exp/
+    | LeftArrApp      l (Exp l) (Exp l)     -- ^ arrow application (from left): /exp/ @-<@ /exp/
+    | RightArrApp     l (Exp l) (Exp l)     -- ^ arrow application (from right): /exp/ @>-@ /exp/
+    | LeftArrHighApp  l (Exp l) (Exp l)     -- ^ higher-order arrow application (from left): /exp/ @-<<@ /exp/
+    | RightArrHighApp l (Exp l) (Exp l)     -- ^ higher-order arrow application (from right): /exp/ @>>-@ /exp/
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -812,12 +818,12 @@ data CallConv l
 #endif
 
 -- | A top level options pragma, preceding the module header.
-data OptionPragma l
-    = LanguagePragma   l [Name l]    -- ^ LANGUAGE pragma
-    | IncludePragma    l String    -- ^ INCLUDE pragma
-    | CFilesPragma     l String    -- ^ CFILES pragma
+data ModulePragma l
+    = LanguagePragma   l [Name l]  -- ^ LANGUAGE pragma
     | OptionsPragma    l (Maybe Tool) String
                         -- ^ OPTIONS pragma, possibly qualified with a tool, e.g. OPTIONS_GHC
+    | AnnModulePragma  l (Annotation l)
+                        -- ^ ANN pragma with module scope
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -879,8 +885,7 @@ data Pat l
     | PLit l (Literal l)                    -- ^ literal constant
     | PNeg l (Pat l)                        -- ^ negated pattern
     | PNPlusK l (Name l) Integer            -- ^ n+k pattern
-    | PInfixApp l (Pat l) (QName l) (Pat l)
-                                            -- ^ pattern with an infix data constructor
+    | PInfixApp l (Pat l) (QName l) (Pat l) -- ^ pattern with an infix data constructor
     | PApp l (QName l) [Pat l]              -- ^ data constructor and argument patterns
     | PTuple l [Pat l]                      -- ^ tuple pattern
     | PList l [Pat l]                       -- ^ list pattern
@@ -890,10 +895,8 @@ data Pat l
     | PWildCard l                           -- ^ wildcard pattern: @_@
     | PIrrPat l (Pat l)                     -- ^ irrefutable pattern: @~/pat/@
     | PatTypeSig l (Pat l) (Type l)         -- ^ pattern with type signature
-    | PViewPat l (Exp l) (Pat l)              -- ^ view patterns of the form @(/exp/ -> /pat/)@
-
+    | PViewPat l (Exp l) (Pat l)            -- ^ view patterns of the form @(/exp/ -> /pat/)@
     | PRPat l [RPat l]                      -- ^ regular list pattern
-
     | PXTag l (XName l) [PXAttr l] (Maybe (Pat l)) [Pat l]
                                             -- ^ XML element pattern
     | PXETag l (XName l) [PXAttr l] (Maybe (Pat l))
@@ -902,11 +905,8 @@ data Pat l
     | PXPatTag l (Pat l)                    -- ^ XML embedded pattern
     | PXRPats  l [RPat l]                   -- ^ XML regular list pattern
     | PExplTypeArg l (QName l) (Type l)     -- ^ Explicit generics style type argument e.g. @f {| Int |} x = ...@
-
     | PQuasiQuote l String String           -- ^ quasi quote pattern: @[$/name/| /string/ |]@
-
     | PBangPat l (Pat l)                    -- ^ strict (bang) pattern: @f !x = ...@
-
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -937,14 +937,14 @@ data RPatOp l
 
 -- | An entity in a regular pattern.
 data RPat l
-    = RPOp l (RPat l) (RPatOp l)
-    | RPEither l (RPat l) (RPat l)
-    | RPSeq l [RPat l]
-    | RPGuard l (Pat l) [Stmt l]
-    | RPCAs l (Name l) (RPat l)
-    | RPAs l (Name l) (RPat l)
-    | RPParen l (RPat l)
-    | RPPat l (Pat l)
+    = RPOp l (RPat l) (RPatOp l)   -- ^ operator pattern, e.g. pat*
+    | RPEither l (RPat l) (RPat l) -- ^ choice pattern, e.g. (1 | 2)
+    | RPSeq l [RPat l]             -- ^ sequence pattern, e.g. (| 1, 2, 3 |)
+    | RPGuard l (Pat l) [Stmt l]   -- ^ guarded pattern, e.g. (| p | p < 3 |)
+    | RPCAs l (Name l) (RPat l)    -- ^ non-linear variable binding, e.g. (foo\@:(1 | 2))*
+    | RPAs l (Name l) (RPat l)     -- ^ linear variable binding, e.g. foo\@(1 | 2)
+    | RPParen l (RPat l)           -- ^ parenthesised pattern, e.g. (2*)
+    | RPPat l (Pat l)              -- ^ an ordinary pattern
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -969,9 +969,9 @@ data Stmt l
     = Generator l (Pat l) (Exp l)
                             -- ^ a generator: /pat/ @<-@ /exp/
     | Qualifier l (Exp l)   -- ^ an /exp/ by itself: in a @do@-expression,
-                            -- an action whose result is discarded;
-                            -- in a list comprehension and pattern guard,
-                            -- a guard expression
+                            --   an action whose result is discarded;
+                            --   in a list comprehension and pattern guard,
+                            --   a guard expression
     | LetStmt l (Binds l)   -- ^ local bindings
     | RecStmt l [Stmt l]    -- ^ a recursive binding group for arrows
 #ifdef __GLASGOW_HASKELL__
@@ -1041,41 +1041,41 @@ data GuardedAlt l
 -- Builtin names.
 
 prelude_mod, main_mod :: l -> ModuleName l
-prelude_mod l         = ModuleName l "Prelude"
-main_mod    l         = ModuleName l "Main"
+prelude_mod l = ModuleName l "Prelude"
+main_mod    l = ModuleName l "Main"
 
 main_name :: l -> Name l
-main_name l       = Ident l "main"
+main_name l = Ident l "main"
 
 unit_con_name :: l -> QName l
-unit_con_name l       = Special l (UnitCon l)
+unit_con_name l = Special l (UnitCon l)
 
 tuple_con_name :: l -> Boxed -> Int -> QName l
-tuple_con_name l b i      = Special l (TupleCon l b (i+1))
+tuple_con_name l b i = Special l (TupleCon l b (i+1))
 
 list_cons_name :: l -> QName l
-list_cons_name l       = Special l (Cons l)
+list_cons_name l = Special l (Cons l)
 
 unboxed_singleton_con_name :: l -> QName l
 unboxed_singleton_con_name l = Special l (UnboxedSingleCon l)
 
 unit_con :: l -> Exp l
-unit_con l         = Con l $ unit_con_name l
+unit_con l = Con l $ unit_con_name l
 
 tuple_con :: l -> Boxed -> Int -> Exp l
-tuple_con l b i       = Con l (tuple_con_name l b i)
+tuple_con l b i = Con l (tuple_con_name l b i)
 
 unboxed_singleton_con :: l -> Exp l
 unboxed_singleton_con l = Con l (unboxed_singleton_con_name l)
 
 as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name :: l -> Name l
-as_name        l   = Ident  l "as"
-qualified_name l   = Ident  l "qualified"
-hiding_name    l   = Ident  l "hiding"
-minus_name     l   = Symbol l "-"
-bang_name      l   = Symbol l "!"
-dot_name       l   = Symbol l "."
-star_name      l   = Symbol l "*"
+as_name        l = Ident  l "as"
+qualified_name l = Ident  l "qualified"
+hiding_name    l = Ident  l "hiding"
+minus_name     l = Symbol l "-"
+bang_name      l = Symbol l "!"
+dot_name       l = Symbol l "."
+star_name      l = Symbol l "*"
 
 export_name, safe_name, unsafe_name, threadsafe_name, stdcall_name, ccall_name :: l -> Name l
 export_name     l = Ident l "export"
@@ -1086,27 +1086,28 @@ stdcall_name    l = Ident l "stdcall"
 ccall_name      l = Ident l "ccall"
 
 unit_tycon_name, fun_tycon_name, list_tycon_name, unboxed_singleton_tycon_name :: l -> QName l
-unit_tycon_name l      = unit_con_name l
-fun_tycon_name  l      = Special l (FunCon l)
-list_tycon_name l      = Special l (ListCon l)
+unit_tycon_name l = unit_con_name l
+fun_tycon_name  l = Special l (FunCon l)
+list_tycon_name l = Special l (ListCon l)
 unboxed_singleton_tycon_name l = Special l (UnboxedSingleCon l)
 
 tuple_tycon_name :: l -> Boxed -> Int -> QName l
-tuple_tycon_name l b i    = tuple_con_name l b i
+tuple_tycon_name l b i = tuple_con_name l b i
 
 unit_tycon, fun_tycon, list_tycon, unboxed_singleton_tycon :: l -> Type l
-unit_tycon l       = TyCon l $ unit_tycon_name l
-fun_tycon  l       = TyCon l $ fun_tycon_name  l
-list_tycon l       = TyCon l $ list_tycon_name l
+unit_tycon l = TyCon l $ unit_tycon_name l
+fun_tycon  l = TyCon l $ fun_tycon_name  l
+list_tycon l = TyCon l $ list_tycon_name l
 unboxed_singleton_tycon l = TyCon l $ unboxed_singleton_tycon_name l
 
 tuple_tycon :: l -> Boxed -> Int -> Type l
-tuple_tycon l b i         = TyCon l (tuple_tycon_name l b i)
+tuple_tycon l b i = TyCon l (tuple_tycon_name l b i)
 
 -----------------------------------------------------------------------------
 -- AST traversal, boiler-plate style
 
-(=~=) :: (Functor a, Eq (a ())) => a l1 -> a l2 -> Bool
+-- | Test if two AST elements are equal modulo annotations.
+(=~=) :: (Annotated a, Eq (a ())) => a l1 -> a l2 -> Bool
 a =~= b = fmap (const ()) a == fmap (const ()) b
 
 instance Functor ModuleName where
@@ -1114,11 +1115,11 @@ instance Functor ModuleName where
 
 instance Functor SpecialCon where
     fmap f sc = case sc of
-        UnitCon l   -> UnitCon (f l)
-        ListCon l   -> ListCon (f l)
-        FunCon  l   -> FunCon  (f l)
+        UnitCon l       -> UnitCon (f l)
+        ListCon l       -> ListCon (f l)
+        FunCon  l       -> FunCon  (f l)
         TupleCon l b n  -> TupleCon (f l) b n
-        Cons l      -> Cons (f l)
+        Cons l          -> Cons (f l)
         UnboxedSingleCon l  -> UnboxedSingleCon (f l)
 
 instance Functor QName where
@@ -1217,11 +1218,17 @@ instance Functor Decl where
         DeprPragmaDecl   l nss            -> DeprPragmaDecl (f l) (map (wp f) nss)
         WarnPragmaDecl   l nss            -> WarnPragmaDecl (f l) (map (wp f) nss)
         InlineSig        l b mact qn      -> InlineSig (f l) b (fmap (fmap f) mact) (fmap f qn)
+        InlineConlikeSig l   mact qn      -> InlineConlikeSig (f l) (fmap (fmap f) mact) (fmap f qn)
         SpecInlineSig    l b mact qn ts   -> SpecInlineSig (f l) b (fmap (fmap f) mact) (fmap f qn) (map (fmap f) ts)
         SpecSig          l        qn ts   -> SpecSig (f l) (fmap f qn) (map (fmap f) ts)
         InstSig          l mcx ih         -> InstSig (f l) (fmap (fmap f) mcx) (fmap f ih)
       where wp f (ns, s) = (map (fmap f) ns, s)
 
+instance Functor Annotation where
+    fmap f (Ann     l n e) = Ann     (f l) (fmap f n) (fmap f e)
+    fmap f (TypeAnn l n e) = TypeAnn (f l) (fmap f n) (fmap f e)
+    fmap f (ModuleAnn l e) = ModuleAnn (f l) (fmap f e)
+
 instance Functor DataOrNew where
     fmap f (DataType l) = DataType (f l)
     fmap f (NewType  l) = NewType  (f l)
@@ -1249,8 +1256,8 @@ instance Functor IPBind where
 instance Functor Match where
     fmap f (Match l n ps rhs bs) =
         Match (f l) (fmap f n) (map (fmap f) ps) (fmap f rhs) (fmap (fmap f) bs)
-    fmap f (InfixMatch l a n b rhs bs) =
-        InfixMatch (f l) (fmap f a) (fmap f n) (fmap f b) (fmap f rhs) (fmap (fmap f) bs)
+    fmap f (InfixMatch l a n ps rhs bs) =
+        InfixMatch (f l) (fmap f a) (fmap f n) (map (fmap f) ps) (fmap f rhs) (fmap (fmap f) bs)
 
 instance Functor QualConDecl where
     fmap f (QualConDecl l mtvs mcx cd) = QualConDecl (f l) (fmap (map (fmap f)) mtvs) (fmap (fmap f) mcx) (fmap f cd)
@@ -1280,7 +1287,7 @@ instance Functor InstDecl where
             -> InsData  (f l) (fmap f dn) (fmap f t)                    (map (fmap f) cds) (fmap (fmap f) ders)
         InsGData  l dn t mk gds ders
             -> InsGData (f l) (fmap f dn) (fmap f t) (fmap (fmap f) mk) (map (fmap f) gds) (fmap (fmap f) ders)
-        InsInline l b mact qn   -> InsInline (f l) b (fmap (fmap f) mact) (fmap f qn)
+--        InsInline l b mact qn   -> InsInline (f l) b (fmap (fmap f) mact) (fmap f qn)
 
 instance Functor BangType where
      fmap f (BangedTy   l t) = BangedTy (f l) (fmap f t)
@@ -1423,11 +1430,10 @@ instance Functor CallConv where
     fmap f (StdCall l) = StdCall (f l)
     fmap f (CCall l) = CCall (f l)
 
-instance Functor OptionPragma where
+instance Functor ModulePragma where
     fmap f (LanguagePragma   l ns) = LanguagePragma (f l) (map (fmap f) ns)
-    fmap f (IncludePragma    l s) = IncludePragma (f l) s
-    fmap f (CFilesPragma     l s) = CFilesPragma (f l) s
     fmap f (OptionsPragma    l mt s) = OptionsPragma (f l) mt s
+    fmap f (AnnModulePragma  l ann) = AnnModulePragma (f l) (fmap f ann)
 
 instance Functor Activation where
     fmap f (ActiveFrom   l k) = ActiveFrom (f l) k
@@ -1531,8 +1537,13 @@ instance Functor GuardedAlt where
 -----------------------------------------------------------------------------
 -- Reading annotations
 
+-- | AST nodes are annotated, and this class allows manipulation of the annotations.
 class Functor ast => Annotated ast where
+    -- | Retrieve the annotation of an AST node.
     ann :: ast l -> l
+    -- | Change the annotation of an AST node. Note that only the annotation of
+    --   the node itself is affected, and not the annotations of any child nodes.
+    --   if all nodes in the AST tree are to be affected, use 'fmap'.
     amap :: (l -> l) -> ast l -> ast l
 
 instance Annotated ModuleName where
@@ -1677,9 +1688,11 @@ instance Annotated Decl where
         DeprPragmaDecl   l nss          -> l
         WarnPragmaDecl   l nss          -> l
         InlineSig        l b act qn     -> l
+        InlineConlikeSig l   act qn     -> l
         SpecSig          l qn ts        -> l
         SpecInlineSig    l b act qn ts  -> l
         InstSig          l cx ih        -> l
+        AnnPragma        l ann          -> l
     amap f decl = case decl of
         TypeDecl     l dh t      -> TypeDecl    (f l) dh t
         TypeFamDecl  l dh mk     -> TypeFamDecl (f l) dh mk
@@ -1706,10 +1719,20 @@ instance Annotated Decl where
         DeprPragmaDecl   l nss           -> DeprPragmaDecl (f l) nss
         WarnPragmaDecl   l nss           -> WarnPragmaDecl (f l) nss
         InlineSig        l b act qn      -> InlineSig (f l) b act qn
+        InlineConlikeSig l   act qn      -> InlineConlikeSig (f l) act qn
         SpecSig          l qn ts         -> SpecSig (f l) qn ts
         SpecInlineSig    l b act qn ts   -> SpecInlineSig (f l) b act qn ts
         InstSig          l mcx ih        -> InstSig (f l) mcx ih
+        AnnPragma        l ann           -> AnnPragma (f l) ann
 
+instance Annotated Annotation where
+    ann (Ann     l n e) = l
+    ann (TypeAnn l n e) = l
+    ann (ModuleAnn l e) = l
+    amap f (Ann     l n e) = Ann     (f l) n e
+    amap f (TypeAnn l n e) = TypeAnn (f l) n e
+    amap f (ModuleAnn l e) = ModuleAnn (f l) e
+    
 instance Annotated DataOrNew where
     ann (DataType l) = l
     ann (NewType  l) = l
@@ -1783,13 +1806,13 @@ instance Annotated InstDecl where
         InsType   l t1 t2       -> l
         InsData   l dn t    cds ders            -> l
         InsGData  l dn t mk gds ders            -> l
-        InsInline l b act qn    -> l
+--        InsInline l b act qn    -> l
     amap f id = case id of
         InsDecl   l d           -> InsDecl (f l) d
         InsType   l t1 t2       -> InsType (f l) t1 t2
         InsData   l dn t    cds ders -> InsData  (f l) dn t    cds ders
         InsGData  l dn t mk gds ders -> InsGData (f l) dn t mk gds ders
-        InsInline l b act qn    -> InsInline (f l) b act qn
+--        InsInline l b act qn    -> InsInline (f l) b act qn
 
 instance Annotated BangType where
      ann (BangedTy   l t) = l
@@ -2028,12 +2051,12 @@ instance Annotated CallConv where
     ann (CCall l) = l
     amap = fmap
 
-instance Annotated OptionPragma where
+instance Annotated ModulePragma where
     ann (LanguagePragma   l ns) = l
-    ann (IncludePragma    l s) = l
-    ann (CFilesPragma     l s) = l
     ann (OptionsPragma    l mt s) = l
+    ann (AnnModulePragma  l a) = l
     amap f (LanguagePragma   l ns) = LanguagePragma (f l) ns
+    amap f (AnnModulePragma  l a) = AnnModulePragma (f l) a
     amap f p = fmap f p
 
 instance Annotated Activation where
