@@ -764,6 +764,7 @@ lexStdToken = do
                   '\n':_ -> do lexNewline
                                str <- lexQQBody
                                return ('\n':str)
+                  []     -> fail "Unexpected end of input while lexing quasi-quoter"
                   _ -> do str <- lexWhile (not . (`elem` "\\|\n"))
                           rest <- lexQQBody
                           return (str++rest)
@@ -1174,8 +1175,8 @@ showToken t = case t of
   Equals            -> "="
   Backslash         -> "\\"
   Bar               -> "|"
-  LeftArrow         -> "->"
-  RightArrow        -> "<-"
+  LeftArrow         -> "<-"
+  RightArrow        -> "->"
   At                -> "@"
   Tilde             -> "~"
   DoubleArrow       -> "=>"
