@@ -334,8 +334,8 @@ instance Pretty Decl where
                 markLine loc $
                 mySep ( [pretty don, ppContext context, pretty name]
                         ++ map pretty nameList)
-                        <+> (myVcat (zipWith (<+>) (equals : repeat (char '|'))
-                                                   (map pretty constrList))
+                  <+> (myVcat (zipWith (<+>) (equals : repeat (char '|'))
+                                             (map pretty constrList))
                         $$$ ppDeriving derives)
 
         pretty (GDataDecl loc don context name nameList optkind gadtList derives) =
@@ -344,7 +344,7 @@ instance Pretty Decl where
                 mySep ( [pretty don, ppContext context, pretty name]
                         ++ map pretty nameList ++ ppOptKind optkind ++ [text "where"])
                         $$$ ppBody classIndent (map pretty gadtList)
-                        $$$ ppDeriving derives
+                        $$$ ppBody letIndent [ppDeriving derives]
 
         pretty (TypeFamDecl loc name nameList optkind) =
                 blankline $
@@ -613,7 +613,7 @@ instance Pretty Activation where
 
 instance Pretty RuleVar where
     pretty (RuleVar n) = pretty n
-    pretty (TypedRuleVar n t) = mySep [pretty n, text "::", pretty t]
+    pretty (TypedRuleVar n t) = parens $ mySep [pretty n, text "::", pretty t]
 
 instance Pretty ModulePragma where
     pretty (LanguagePragma _ ns) =
@@ -718,11 +718,11 @@ ppForall (Just vs) =    myFsep (text "forall" : map pretty vs ++ [char '.'])
 ---------------------------- Kinds ----------------------------
 
 instance Pretty Kind where
-        pretty KindStar      = text "*"
-        pretty KindBang      = text "!"
-        pretty (KindFn a b)  = myFsep [pretty a, text "->", pretty b]
-        pretty (KindParen k) = parens $ pretty k
-        pretty (KindVar n)   = pretty n
+        prettyPrec _ KindStar      = text "*"
+        prettyPrec _ KindBang      = text "!"
+        prettyPrec n (KindFn a b)  = parensIf (n > 0) $ myFsep [prettyPrec 1 a, text "->", pretty b]
+        prettyPrec _ (KindParen k) = parens $ pretty k
+        prettyPrec _ (KindVar n)   = pretty n
 
 ppOptKind :: Maybe Kind -> [Doc]
 ppOptKind Nothing  = []
@@ -763,28 +763,28 @@ instance Pretty Literal where
 instance Pretty Exp where
         prettyPrec _ (Lit l) = pretty l
         -- lambda stuff
-        prettyPrec p (InfixApp a op b) = parensIf (p > 0) $ myFsep [pretty a, pretty op, pretty b]
-        prettyPrec _ (NegApp e) = parens $ myFsep [char '-', pretty e]
-        prettyPrec p (App a b) = parensIf (p > 0) $ myFsep [pretty a, prettyPrec 1 b]
-        prettyPrec p (Lambda _loc expList ppBody) = parensIf (p > 0) $ myFsep $
-                char '\\' : map pretty expList ++ [text "->", pretty ppBody]
+        prettyPrec p (InfixApp a op b) = parensIf (p > 2) $ myFsep [prettyPrec 2 a, pretty op, prettyPrec 1 b]
+        prettyPrec p (NegApp e) = parensIf (p > 0) $ char '-' <> prettyPrec 4 e
+        prettyPrec p (App a b) = parensIf (p > 3) $ myFsep [prettyPrec 3 a, prettyPrec 4 b]
+        prettyPrec p (Lambda _loc patList ppBody) = parensIf (p > 1) $ myFsep $
+                char '\\' : map pretty patList ++ [text "->", pretty ppBody]
         -- keywords
         -- two cases for lets
         prettyPrec p (Let (BDecls declList) letBody) =
-                parensIf (p > 0) $ ppLetExp declList letBody
+                parensIf (p > 1) $ ppLetExp declList letBody
         prettyPrec p (Let (IPBinds bindList) letBody) =
-                parensIf (p > 0) $ ppLetExp bindList letBody
+                parensIf (p > 1) $ ppLetExp bindList letBody
 
-        prettyPrec p (If cond thenexp elsexp) = parensIf (p > 0) $
+        prettyPrec p (If cond thenexp elsexp) = parensIf (p > 1) $
                 myFsep [text "if", pretty cond,
                         text "then", pretty thenexp,
                         text "else", pretty elsexp]
-        prettyPrec p (Case cond altList) = parensIf (p > 0) $
+        prettyPrec p (Case cond altList) = parensIf (p > 1) $
                 myFsep [text "case", pretty cond, text "of"]
                 $$$ ppBody caseIndent (map pretty altList)
-        prettyPrec p (Do stmtList) = parensIf (p > 0) $ 
+        prettyPrec p (Do stmtList) = parensIf (p > 1) $ 
                 text "do" $$$ ppBody doIndent (map pretty stmtList)
-        prettyPrec p (MDo stmtList) = parensIf (p > 0) $ 
+        prettyPrec p (MDo stmtList) = parensIf (p > 1) $ 
                 text "mdo" $$$ ppBody doIndent (map pretty stmtList)
         -- Constructors & Vars
         prettyPrec _ (Var name) = pretty name
@@ -846,7 +846,7 @@ instance Pretty Exp where
                             int a, char ':', int b, char '-',
                             int c, char ':', int d, text "#-}", pretty e]
         -- Arrows
-        prettyPrec p (Proc _ pat e) = parensIf (p > 0) $ myFsep $ [text "proc", pretty pat, text "->", pretty e]
+        prettyPrec p (Proc _ pat e) = parensIf (p > 1) $ myFsep $ [text "proc", pretty pat, text "->", pretty e]
         prettyPrec p (LeftArrApp l r)      = parensIf (p > 0) $ myFsep $ [pretty l, text "-<",  pretty r]
         prettyPrec p (RightArrApp l r)     = parensIf (p > 0) $ myFsep $ [pretty l, text ">-",  pretty r]
         prettyPrec p (LeftArrHighApp l r)  = parensIf (p > 0) $ myFsep $ [pretty l, text "-<<", pretty r]
@@ -889,25 +889,25 @@ instance Pretty Splice where
 instance Pretty Pat where
         prettyPrec _ (PVar name) = pretty name
         prettyPrec _ (PLit lit) = pretty lit
-        prettyPrec _ (PNeg p) = myFsep [char '-', pretty p]
+        prettyPrec p (PNeg pat) = parensIf (p > 0) $ myFsep [char '-', pretty pat]
         prettyPrec p (PInfixApp a op b) = parensIf (p > 0) $
-                myFsep [pretty a, pretty (QConOp op), pretty b]
-        prettyPrec p (PApp n ps) = parensIf (p > 1) $
-                myFsep (pretty n : map pretty ps)
+                myFsep [prettyPrec 1 a, pretty (QConOp op), prettyPrec 1 b]
+        prettyPrec p (PApp n ps) = parensIf (p > 1 && not (null ps)) $
+                myFsep (pretty n : map (prettyPrec 2) ps)
         prettyPrec _ (PTuple ps) = parenList . map pretty $ ps
         prettyPrec _ (PList ps) =
                 bracketList . punctuate comma . map pretty $ ps
-        prettyPrec _ (PParen p) = parens . pretty $ p
+        prettyPrec _ (PParen pat) = parens . pretty $ pat
         prettyPrec _ (PRec c fields) =
                 pretty c <> (braceList . map pretty $ fields)
         -- special case that would otherwise be buggy
         prettyPrec _ (PAsPat name (PIrrPat pat)) =
-                myFsep [pretty name <> char '@', char '~' <> pretty pat]
+                myFsep [pretty name <> char '@', char '~' <> prettyPrec 2 pat]
         prettyPrec _ (PAsPat name pat) =
-                hcat [pretty name, char '@', pretty pat]
+                hcat [pretty name, char '@', prettyPrec 2 pat]
         prettyPrec _ PWildCard = char '_'
-        prettyPrec _ (PIrrPat pat) = char '~' <> pretty pat
-        prettyPrec _ (PatTypeSig _pos pat ty) =
+        prettyPrec _ (PIrrPat pat) = char '~' <> prettyPrec 2 pat
+        prettyPrec p (PatTypeSig _pos pat ty) = parensIf (p > 0) $
                 myFsep [pretty pat, text "::", pretty ty]
         prettyPrec p (PViewPat e pat) = parensIf (p > 0) $
                 myFsep [pretty e, text "->", pretty pat]
@@ -934,7 +934,7 @@ instance Pretty Pat where
         prettyPrec _ (PExplTypeArg qn t) =
                 myFsep [pretty qn, text "{|", pretty t, text "|}"]
         -- BangPatterns
-        prettyPrec _ (PBangPat p) = text "!" <> pretty p
+        prettyPrec _ (PBangPat pat) = text "!" <> prettyPrec 2 pat
 
 instance Pretty PXAttr where
         pretty (PXAttr n p) =
